package com.restlet.client.log.impl;

import com.restlet.client.function.Function;
import com.restlet.client.log.LogService;

import static com.restlet.client.log.LogService.Severity.Debug;
import static com.restlet.client.log.LogService.Severity.Error;
import static com.restlet.client.log.LogService.Severity.Fine;
import static com.restlet.client.log.LogService.Severity.Info;
import static com.restlet.client.log.LogService.Severity.Warning;
import static com.restlet.client.utils.FunctionalUtils.ifNotNull;

public class StdOutLogService implements LogService {

    private void log(final Severity severity, final Class<?> clazz, final String message, final Throwable error) {
        System.out.println(new StringBuilder()
                .append('[')
                .append(severity.name().toUpperCase())
                .append(']')
                .append(ifNotNull(clazz, new Function<Class<?>, String>() {
                    @Override
                    public String apply(final Class<?> clazz) {
                        return clazz.getSimpleName() + ": ";
                    }
                }, ""))
                .append(message)
                .append(ifNotNull(error, new Function<Throwable, String>() {
                    @Override
                    public String apply(final Throwable error) {
                        return error.getMessage();
                    }
                }, ""))
                .toString());
    }

    @Override
    public void debug(final Class<?> clazz, final String message) {
        log(Debug, clazz, message, null);
    }

    @Override
    public void debug(final Class<?> clazz, final String message, final Throwable e) {
        log(Debug, clazz, message, e);
    }

    @Override
    public void fine(final Class<?> clazz, final String message) {
        log(Fine, clazz, message, null);
    }

    @Override
    public void error(final String message) {
        log(Error, null, message, null);
    }

    @Override
    public void error(final String message, final Throwable e) {
        log(Error, null, message, e);
    }

    @Override
    public void info(final String message) {
        log(Info, null, message, null);
    }

    @Override
    public void warning(final String message) {
        log(Warning, null, message, null);
    }

    @Override
    public void log(final Severity severity, final String message) {
        log(severity, null, message, null);
    }
}
