package com.restlet.client.model;

import com.restlet.client.function.Function;
import com.restlet.client.function.Predicate;
import com.restlet.client.utils.Sequence;

import java.util.Set;

public enum ImportPolicy {
    Override("Override", true),
    Update("Update", true),
    Preserve("Preserve", true),
    Sharing("Sharing", false),
    Synchronization("Synchronization", false);

    public final String name;
    public final boolean isFileImportPolicy;

    ImportPolicy(final String name, boolean isFileImportPolicy) {
        this.name = name;
        this.isFileImportPolicy = isFileImportPolicy;
    }

    public static Set<ImportPolicy> getFileImportPolicies() {
        return Sequence.of(values())
                .filter(isFileImportPolicyFilter())
                .toSet();
    }

    private static Predicate<ImportPolicy> isFileImportPolicyFilter() {
        return new Predicate<ImportPolicy>() {
            @Override
            public boolean test(ImportPolicy importPolicy) {
                return importPolicy.isFileImportPolicy;
            }
        };
    }

    public Function<EntityTreeNode, String> identifierRetriever() {
        return new Function<EntityTreeNode, String>() {
            @Override
            public String apply(final EntityTreeNode treeNode) {
                return isFileImportPolicy
                        ? treeNode.entity.getName()
                        : treeNode.entity.getId();
            }
        };
    }

    public Function<String, EntityTreeNode> childRetriever(final EntityTreeNode parent) {
        return new Function<String, EntityTreeNode>() {
            @Override
            public EntityTreeNode apply(final String identifier) {
                return isFileImportPolicy
                        ? parent.retrieveChildByName(identifier)
                        : parent.retrieveChildById(identifier);
            }
        };
    }
}
