package com.restlet.client.model;

import com.restlet.client.function.Consumer;
import com.restlet.client.function.Function;
import com.restlet.client.utils.EntityUtils;
import com.restlet.client.utils.FunctionalUtils;
import com.restlet.client.utils.Holder;
import com.restlet.client.utils.Sequence;
import com.restlet.client.utils.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public enum UpdateStatus {
    REQUEST_ORDER_WARNING {
        @Override
        public String generateHtmlErrorMessage(List<EntityTreeNode> incriminatedElements) {
            return StringUtils.format("<p>An issue has been detected with your data.<br>Please double check that the requests of the following imported scenarios are correctly ordered:</p><ul>%s</ul>",
                    StringUtils.joiner(Sequence.of(incriminatedElements)
                            .map(new Function<EntityTreeNode, String>() {
                                @Override
                                public String apply(EntityTreeNode entityTreeNode) {
                                    final List<String> names = new ArrayList<>();
                                    EntityUtils.walkParents(entityTreeNode.entity,
                                            new Consumer<EntityTo>() {
                                                @Override
                                                public void consume(EntityTo entityTo) {
                                                    names.add(0, "\"" + entityTo.getName() + "\"");
                                                }
                                            }
                                    );
                                    return StringUtils.format("<li>%s</li>", StringUtils.joiner(names).joinWith(" / "));
                                }
                            })
                            .toList()).joinWith("")
            );
        }
    };

    public abstract String generateHtmlErrorMessage(final List<EntityTreeNode> incriminatedElements);

    public static Map<UpdateStatus, List<EntityTreeNode>> indexTreeNodesByUpdateStatus(Holder<EntityTreeNode> persistedNodeHolder) {
        final Map<UpdateStatus, List<EntityTreeNode>> map = Sequence.of(UpdateStatus.values())
                .toMap(FunctionalUtils.<UpdateStatus>identity(),
                        new Function<UpdateStatus, List<EntityTreeNode>>() {
                            @Override
                            public List<EntityTreeNode> apply(UpdateStatus updateStatus) {
                                return new ArrayList<>();
                            }
                        });

        persistedNodeHolder.value.walk(new Consumer<EntityTreeNode>() {
            @Override
            public void consume(EntityTreeNode node) {
                if (node.updateStatus != null) {
                    map.get(node.updateStatus).add(node);
                }
            }
        });
        return map;
    }

    ;
}
