package com.restlet.client.script.runtime.impl;

import com.restlet.client.net.MessageHeaderTo;
import com.restlet.client.platform.json.JsonArray;
import com.restlet.client.platform.json.JsonEngine;
import com.restlet.client.platform.json.JsonObject;
import com.restlet.client.platform.json.JsonValue;
import com.restlet.client.utils.FunctionalUtils;

import java.util.List;

import static com.restlet.client.utils.FunctionalUtils.ifNotNull;
import static com.restlet.client.utils.Objects.isNullOrEmpty;
import static com.restlet.client.utils.StringUtils.isBlank;

public class ResponseJSONReference {

    private final JsonObject root;
    private final JsonEngine jsonEngine;

    public ResponseJSONReference(JsonObject root, JsonEngine jsonEngine) {
        this.root = root;
        this.jsonEngine = jsonEngine;
    }

    public void setHeaders(final List<MessageHeaderTo> headers) {
        if (!isNullOrEmpty(headers)) {
            JsonObject refHeaders = jsonEngine.newJsonObject();
            for (MessageHeaderTo reqHeader : headers) {
                if (reqHeader == null || isBlank(reqHeader.getName()) || reqHeader.getValue() == null) {
                    continue;
                }

                JsonValue refHeader = refHeaders.get(reqHeader.getName().toLowerCase());
                if (refHeader == null) {
                    refHeaders.put(reqHeader.getName().toLowerCase(), jsonEngine.newJsonString(reqHeader.getValue()));

                } else if (refHeader.isArray() != null) {
                    refHeader.isArray().set(refHeader.isArray().size(), jsonEngine.newJsonString(reqHeader.getValue()));

                } else if (refHeader.isString() != null) {
                    JsonArray refHeadersArray = jsonEngine.newJsonArray();
                    refHeadersArray.set(0, refHeader);
                    refHeadersArray.set(1, jsonEngine.newJsonString(reqHeader.getValue()));
                    refHeaders.put(reqHeader.getName().toLowerCase(), refHeadersArray);
                }
            }
            root.put("headers", refHeaders);
        } else {
            root.put("headers", null);
        }
    }

    JsonValue asJSON() {
        return root;
    }

    public void setStatus(Integer statusCode, String statusMessage) {
        if (statusCode != null || !isBlank(statusMessage)) {
            JsonObject status = jsonEngine.newJsonObject();
            if (statusCode != null) {
                status.put("code", jsonEngine.newJsonNumber(statusCode));
            }
            if (!isBlank(statusMessage)) {
                status.put("message", jsonEngine.newJsonString(statusMessage));
            }
            root.put("status", status);
        }
    }

    public void setBody(String text) {
        if (!isBlank(text)) {
            try {
                JsonValue json = jsonEngine.fromJson(text);
                root.put("body", ifNotNull(json, FunctionalUtils.<JsonValue>identity(), jsonEngine.newJsonString(text)));

            } catch (Throwable e) {
                root.put("body", jsonEngine.newJsonString(text));
            }
        }
    }
}
