package com.restlet.client.script.value;

import com.restlet.client.async.Promise;
import com.restlet.client.async.Promises;
import com.restlet.client.platform.json.JsonEngine;
import com.restlet.client.platform.json.JsonValue;
import com.restlet.client.script.runtime.ScriptValue;

public class ScriptJsonValue implements ScriptValue {

    private final JsonEngine engine;
    private final JsonValue value;

    public ScriptJsonValue(JsonValue value, JsonEngine engine) {
        super();
        this.value = value;
        this.engine = engine;
    }

    @Override
    public String toString() {
        if (value == null || value.isNull() != null) {
            return null;
        }
        if (value.isString() != null) {
            return value.isString().stringValue();
        }

        return value.toString();
    }

    @Override
    public Promise<ScriptValue> select(String path) {
        if (value == null) {
            return Promises.of();
        }

        if (value.isObject() != null) {
            JsonValue jsonValue = value.isObject().get(path);
            if (jsonValue != null) {
                return Promises.of((ScriptValue) new ScriptJsonValue(jsonValue, engine));
            }

        } else if (value.isArray() != null) {
            try {

                Integer index = Integer.valueOf(path);
                if (index != null && index >= 0 && index < value.isArray().size()) {

                    JsonValue jsonValue = value.isArray().get(index);
                    if (jsonValue != null) {
                        return Promises.of((ScriptValue) new ScriptJsonValue(jsonValue, engine));
                    }
                }

            } catch (NumberFormatException e) {
                // do nothing
            }
        } else if (value.isString() != null) {
            return (new ScriptStringValue(value.toString(), engine)).select(path);

        }
        return Promises.of();
    }

    public JsonValue getJsonValue() {
        return value;
    }
}
