package com.restlet.client.tests.asserts;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import static java.util.Arrays.asList;

public class AssertionResultTo implements AssertionTo {

    public static final String ASSERTION_PATH_STATUS_MESSAGE = "message";
    public static final String ASSERTION_PATH_STATUS_CODE = "code";
    public static final String ASSERTION_PATH_RESPONSE_BODY_CONTENT = "content";
    public static final String ASSERTION_PATH_RESPONSE_BODY_LENGTH = "length";
    public static final String ASSERTION_PATH_RESPONSE_LATENCY = "latency";

    private State state;
    private final AssertionTo assertion;
    private final List<AssertionResultMessage> messages = new ArrayList<>();
    private String evaluatedPath;
    private String evaluatedValue;
    private String actualValue;

    public AssertionResultTo(AssertionTo assertion) {
        this.assertion = assertion;
    }

    @Override
    public Boolean isEnabled() {
        return assertion.isEnabled();
    }

    @Override
    public void setEnabled(Boolean enabled) {
        assertion.setEnabled(enabled);
    }

    @Override
    public Subject getSubject() {
        return assertion.getSubject();
    }

    @Override
    public void setSubject(Subject subject) {
        assertion.setSubject(subject);
    }

    @Override
    public String getPath() {
        return assertion.getPath();
    }

    @Override
    public void setPath(String path) {
        assertion.setPath(path);
    }

    @Override
    public Comparison getComparison() {
        return assertion.getComparison();
    }

    @Override
    public void setComparison(Comparison comparison) {
        assertion.setComparison(comparison);
    }

    @Override
    public String getValue() {
        return assertion.getValue();
    }

    @Override
    public void setValue(String value) {
        assertion.setValue(value);
    }

    public final AssertionTo getAssertion() {
        return assertion;
    }

    public State getState() {
        return state;
    }

    public void setState(State state) {
        this.state = state;
    }

    public List<AssertionResultMessage> getMessages() {
        return messages;
    }

    public List<AssertionResultMessage> getMessages(State state) {
        List<AssertionResultMessage> messagesByState = new ArrayList<>();
        for (AssertionResultMessage message : messages) {
            if (message.state == state) {
                messagesByState.add(message);
            }
        }
        return messagesByState;
    }

    public void addFailureMessage(String message) {
        messages.add(new AssertionResultMessage(State.Failure, message));
    }

    public void addErrorMessage(String message) {
        messages.add(new AssertionResultMessage(State.Error, message));
    }

    public void addWarningMessage(String message) {
        messages.add(new AssertionResultMessage(State.Warning, message));
    }

    public String getEvaluatedPath() {
        return evaluatedPath;
    }

    public void setEvaluatedPath(String evaluatedPath) {
        this.evaluatedPath = evaluatedPath;
    }

    public String getEvaluatedValue() {
        return evaluatedValue;
    }

    public void setEvaluatedValue(String evaluatedValue) {
        this.evaluatedValue = evaluatedValue;
    }

    public String getActualValue() {
        return actualValue;
    }

    public AssertionResultTo setActualValue(String actualValue) {
        this.actualValue = actualValue;
        return this;
    }

    public enum State {
        Ok, Failure, Error, Warning;

        private static final List<State> ORDERED_STATES = asList(Error, Failure, Warning, Ok);

        public static Comparator<State> getStateComparator() {
            return new Comparator<State>() {
                @Override
                public int compare(State state1, State state2) {
                    Integer state1Index = ORDERED_STATES.indexOf(state1);
                    if (state1Index == -1) {
                        throw new RuntimeException("State " + state1 + " is not in the ordered list of states.");
                    }

                    Integer state2Index = ORDERED_STATES.indexOf(state2);
                    if (state2Index == -1) {
                        throw new RuntimeException("State " + state2 + " is not in the ordered list of states.");
                    }

                    return -state1Index.compareTo(state2Index);
                }
            };
        }
    }

    public static class AssertionResultMessage {
        public State state;
        public String message;

        AssertionResultMessage(State state, String message) {
            this.state = state;
            this.message = message;
        }
    }
}
