package com.restlet.client.utils;

import com.restlet.client.function.BiFunction;
import com.restlet.client.function.Function;
import com.restlet.client.model.EntityTo;
import com.restlet.client.model.context.ContextVariableTo;
import com.restlet.client.net.http.request.HttpRequestTo;
import com.restlet.client.tests.ScenarioTo;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import static com.restlet.client.utils.EntityUtils.indexOf;
import static com.restlet.client.utils.StringUtils.isBlank;

public class ComparatorUtils {

    public static void sortHttpRequests(ScenarioTo scenarioTo) {
        if (scenarioTo == null) {
            return;
        }
        sortAsScenarioRequests(scenarioTo.getHttpRequests(), scenarioTo, new Function<HttpRequestTo, String>() {
            @Override
            public String apply(HttpRequestTo requestTo) {
                return requestTo.getName();
            }
        });
    }

    /**
     * According to the scenario requests order, sorts list where elements as similar to HttpRequestTo instances.
     * At least each element is able to compute the name of one of the scenario's HTTP request.
     */
    public static <T> void sortAsScenarioRequests(List<T> toSort, final ScenarioTo scenarioTo, final Function<T, String> nameMapper) {
        if (scenarioTo == null || toSort == null) {
            return;
        }
        Collections.sort(toSort, new Comparator<T>() {
            @Override
            public int compare(T o1, T o2) {
                int index1 = indexOf(scenarioTo, nameMapper.apply(o1));
                int index2 = indexOf(scenarioTo, nameMapper.apply(o2));

                return index1 - index2;
            }
        });
    }

    public static void sortContextVariables(final List<ContextVariableTo> variables) {
        Collections.sort(variables, new Comparator<ContextVariableTo>() {
            @Override
            public int compare(ContextVariableTo variable1, ContextVariableTo variable2) {
                if (isBlank(variable1.getName()) && !isBlank(variable2.getName())) {
                    return 1;
                }
                if (!isBlank(variable1.getName()) && isBlank(variable2.getName())) {
                    return -1;
                }
                return variable1.getName().compareTo(variable2.getName());
            }
        });
    }

    public static BiFunction<EntityTo, EntityTo, Boolean> equalityMatcherById = new BiFunction<EntityTo, EntityTo, Boolean>() {
        @Override
        public Boolean apply(EntityTo e1, EntityTo e2) {
            return java.util.Objects.equals(e1.getId(), e2.getId());
        }
    };

}
