package com.restlet.client.utils;


public class IntegerRange {

    final Integer lowerBound;
    final Integer upperBound;

    private IntegerRange(Integer lowerBound, Integer upperBound) {
        this.lowerBound = lowerBound;
        this.upperBound = upperBound;
    }

    public static IntegerRange inclusive(Integer lower, Integer upper) {
        return new IntegerRange(lower, upper);
    }

    public boolean isEmpty() {
        return (lowerBound == null && upperBound == null) || (lowerBound != null && lowerBound.equals(upperBound));
    }

    public Integer lowerEndpoint() {
        return lowerBound;
    }

    public Integer upperEndpoint() {
        return upperBound;
    }

    public boolean containsInteger(int value) {
        return (lowerBound == null || value >= lowerBound) && (upperBound == null || value <= upperBound);
    }

    public boolean equals(Object obj) {
        if (obj == this) {
            return true;

        } else if (obj == null || obj.getClass() != getClass()) {
            return false;

        } else {
            IntegerRange range = (IntegerRange) obj;

            if (lowerBound == null) {
                if (range.lowerBound != null) {
                    return false;
                }

            } else if (range.lowerBound == null) {
                return false;
            }

            if (upperBound == null) {
                if (range.upperBound != null) {
                    return false;
                }

            } else if (range.upperBound == null) {
                return false;
            }

            return ((lowerBound == null && range.lowerBound == null) || lowerBound.equals(range.lowerBound)) &&
                    ((upperBound == null && range.upperBound == null) || upperBound.equals(range.upperBound)
                    );
        }
    }

    public int hashCode() {
        int result = 17;
        result = 37 * result + getClass().hashCode();
        result = 37 * result + (lowerBound != null ? lowerBound.hashCode() : "null".hashCode());
        result = 37 * result + (upperBound != null ? upperBound.hashCode() : "null".hashCode());
        return result;
    }

    public String toString() {
        StringBuilder buf = new StringBuilder(32);
        buf.append(this.getClass().getSimpleName() + "[");
        buf.append(lowerBound);
        buf.append(',');
        buf.append(upperBound);
        buf.append(']');
        return buf.toString();
    }
}
