package com.restlet.client.utils;

import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;

public class Iterables {

    public static <T> Maybe<T> getFirst(final Iterable<T> iterable) {
        if (iterable == null) {
            return Maybe.empty();
        }

        final Iterator<T> iterator = iterable.iterator();
        if (iterator.hasNext()) {
            return Maybe.of(iterator.next());
        }
        return Maybe.empty();
    }

    public static <T> T getLast(Iterable<T> iterable) {
        if (iterable instanceof List) {
            List<T> list = (List<T>) iterable;
            if (list.isEmpty()) {
                throw new NoSuchElementException();
            } else {
                return getLastInNonemptyList(list);
            }
        } else {
            return getLast(iterable.iterator());
        }
    }

    private static <T> T getLastInNonemptyList(List<T> list) {
        return list.get(list.size() - 1);
    }

    public static <T> T getLast(Iterator<T> iterator) {
        T current;
        do {
            current = iterator.next();
        } while (iterator.hasNext());

        return current;
    }
}
