package com.restlet.client.utils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

public class ListUtils {

    public static <C> List<C> reverse(List<C> list) {
        return Sequence.of(list)
                .reverse()
                .toList();
    }

    public static <T extends Comparable<T>> List<T> sort(Collection<T> versions) {
        final List<T> orderedVersions = new ArrayList<>(versions);
        Collections.sort(orderedVersions);
        return orderedVersions;
    }

    @SafeVarargs
    public static <T> List<T> join(List<T>... lists) {
        final List<T> joinedLists = new ArrayList<>();
        for (List<T> list : lists) {
            joinedLists.addAll(list);
        }
        return joinedLists;
    }

    public static <T> T last(List<T> elements) {
        final Maybe<T> maybeLast = Sequence.of(elements)
                .takeRight(1)
                .first();
        return maybeLast.isPresent() ? maybeLast.get() : null;
    }

    public static <T> T first(Collection<T> elements) {
        final Maybe<T> maybeFirst = Sequence.of(elements)
                .takeLeft(1)
                .first();
        return maybeFirst.isPresent() ? maybeFirst.get() : null;
    }

    public static <T> List<T> newFilledList(int size, T fillWith) {
        final List<T> result = new ArrayList<>(size);

        for (int i = 0; i < size; i++) {
            result.add(fillWith);
        }

        return result;
    }

    public static <T> List<T> add(List<T> input, int index, T element) {
        if (index < 0 || index > input.size()) {
            throw new IndexOutOfBoundsException();
        }
        if (index == input.size()) {
            input.add(element);
            return input;
        }

        List<T> endOfList = new ArrayList<>(input.subList(index, input.size()));
        while (input.size() > index) {
            input.remove(index);
        }
        input.add(element);
        input.addAll(endOfList);
        return input;
    }
}
